@@ -12,15 +12,12 @@ except ImportError:
 from . import ThreeMFWorkspaceReader
 
 from UM.i18n import i18nCatalog
-from UM.Platform import Platform
+
 catalog = i18nCatalog("cura")
 
+
 def getMetaData() -> Dict:
-    # Workarround for osx not supporting double file extensions correctly.
-    if Platform.isOSX():
-        workspace_extension = "3mf"
-    else:
-        workspace_extension = "curaproject.3mf"
+    workspace_extension = "3mf"
 
     metaData = {}
     if "3MFReader.ThreeMFReader" in sys.modules:
