@@ -4,33 +4,42 @@
 import json
 import os
 import tempfile
-import platform
 from typing import cast, Any, Dict, List, Set, TYPE_CHECKING, Tuple, Optional, Union
 
-from PyQt5.QtCore import QUrl, QObject, pyqtProperty, pyqtSignal, pyqtSlot
+from PyQt5.QtCore import QObject, pyqtProperty, pyqtSignal, pyqtSlot
 from PyQt5.QtNetwork import QNetworkAccessManager, QNetworkRequest, QNetworkReply
 
+from UM.Extension import Extension
 from UM.Logger import Logger
 from UM.PluginRegistry import PluginRegistry
-from UM.Extension import Extension
-from UM.i18n import i18nCatalog
+from UM.TaskManagement.HttpRequestScope import JsonDecoratorScope
 from UM.Version import Version
-
+from UM.i18n import i18nCatalog
 from cura import ApplicationMetadata
-from cura import UltimakerCloudAuthentication
 from cura.CuraApplication import CuraApplication
 from cura.Machines.ContainerTree import ContainerTree
-
+from cura.UltimakerCloud.UltimakerCloudScope import UltimakerCloudScope
 from .AuthorsModel import AuthorsModel
+from .CloudApiModel import CloudApiModel
+from .CloudSync.LicenseModel import LicenseModel
 from .PackagesModel import PackagesModel
 
 if TYPE_CHECKING:
+    from UM.TaskManagement.HttpRequestData import HttpRequestData
     from cura.Settings.GlobalStack import GlobalStack
 
 i18n_catalog = i18nCatalog("cura")
 
+DEFAULT_MARKETPLACE_ROOT = "https://marketplace.ultimaker.com"  # type: str
 
-##  The Toolbox class is responsible of communicating with the server through the API
+try:
+    from cura.CuraVersion import CuraMarketplaceRoot
+except ImportError:
+    CuraMarketplaceRoot = DEFAULT_MARKETPLACE_ROOT
+
+# todo Remove license and download dialog, use SyncOrchestrator instead
+
+##  Provides a marketplace for users to download plugins an materials
 class Toolbox(QObject, Extension):
     def __init__(self, application: CuraApplication) -> None:
         super().__init__()
@@ -38,47 +47,49 @@ class Toolbox(QObject, Extension):
         self._application = application  # type: CuraApplication
 
         self._sdk_version = ApplicationMetadata.CuraSDKVersion  # type: Union[str, int]
-        self._cloud_api_version = UltimakerCloudAuthentication.CuraCloudAPIVersion  # type: str
-        self._cloud_api_root = UltimakerCloudAuthentication.CuraCloudAPIRoot  # type: str
-        self._api_url = None  # type: Optional[str]
 
         # Network:
-        self._download_request = None  # type: Optional[QNetworkRequest]
-        self._download_reply = None  # type: Optional[QNetworkReply]
+        self._download_request_data = None  # type: Optional[HttpRequestData]
         self._download_progress = 0  # type: float
         self._is_downloading = False  # type: bool
-        self._network_manager = None  # type: Optional[QNetworkAccessManager]
-        self._request_headers = []  # type: List[Tuple[bytes, bytes]]
-        self._updateRequestHeader()
+        self._cloud_scope = UltimakerCloudScope(application)  # type: UltimakerCloudScope
+        self._json_scope = JsonDecoratorScope(self._cloud_scope)  # type: JsonDecoratorScope
 
-        self._request_urls = {}  # type: Dict[str, QUrl]
+        self._request_urls = {}  # type: Dict[str, str]
         self._to_update = []  # type: List[str] # Package_ids that are waiting to be updated
         self._old_plugin_ids = set()  # type: Set[str]
         self._old_plugin_metadata = dict()  # type: Dict[str, Dict[str, Any]]
 
         # The responses as given by the server parsed to a list.
         self._server_response_data = {
-            "authors":             [],
-            "packages":            [],
-            "updates":             [],
+            "authors":              [],
+            "packages":             [],
+            "updates":              []
         }  # type: Dict[str, List[Any]]
 
         # Models:
         self._models = {
-            "authors":             AuthorsModel(self),
-            "packages":            PackagesModel(self),
-            "updates":             PackagesModel(self),
+            "authors":              AuthorsModel(self),
+            "packages":             PackagesModel(self),
+            "updates":              PackagesModel(self)
         }  # type: Dict[str, Union[AuthorsModel, PackagesModel]]
 
         self._plugins_showcase_model = PackagesModel(self)
         self._plugins_available_model = PackagesModel(self)
         self._plugins_installed_model = PackagesModel(self)
-
+        self._plugins_installed_model.setFilter({"is_bundled": "False"})
+        self._plugins_bundled_model = PackagesModel(self)
+        self._plugins_bundled_model.setFilter({"is_bundled": "True"})
         self._materials_showcase_model = AuthorsModel(self)
         self._materials_available_model = AuthorsModel(self)
         self._materials_installed_model = PackagesModel(self)
+        self._materials_installed_model.setFilter({"is_bundled": "False"})
+        self._materials_bundled_model = PackagesModel(self)
+        self._materials_bundled_model.setFilter({"is_bundled": "True"})
         self._materials_generic_model = PackagesModel(self)
 
+        self._license_model = LicenseModel()
+
         # These properties are for keeping track of the UI state:
         # ----------------------------------------------------------------------
         # View category defines which filter to use, and therefore effectively
@@ -88,7 +99,7 @@ class Toolbox(QObject, Extension):
 
         # View page defines which type of page layout to use. For example,
         # possible values include "overview", "detail" or "author".
-        self._view_page = "loading"  # type: str
+        self._view_page = "welcome"  # type: str
 
         # Active package refers to which package is currently being downloaded,
         # installed, or otherwise modified.
@@ -101,14 +112,9 @@ class Toolbox(QObject, Extension):
         self._restart_required = False  # type: bool
 
         # variables for the license agreement dialog
-        self._license_dialog_plugin_name = ""  # type: str
-        self._license_dialog_license_content = ""  # type: str
         self._license_dialog_plugin_file_location = ""  # type: str
-        self._restart_dialog_message = ""  # type: str
 
         self._application.initializationFinished.connect(self._onAppInitialized)
-        self._application.getCuraAPI().account.loginStateChanged.connect(self._updateRequestHeader)
-        self._application.getCuraAPI().account.accessTokenChanged.connect(self._updateRequestHeader)
 
     # Signals:
     # --------------------------------------------------------------------------
@@ -126,23 +132,17 @@ class Toolbox(QObject, Extension):
     filterChanged = pyqtSignal()
     metadataChanged = pyqtSignal()
     showLicenseDialog = pyqtSignal()
+    closeLicenseDialog = pyqtSignal()
     uninstallVariablesChanged = pyqtSignal()
 
-    def _updateRequestHeader(self):
-        self._request_headers = [
-            (b"User-Agent",
-            str.encode(
-                "%s/%s (%s %s)" % (
-                    self._application.getApplicationName(),
-                    self._application.getVersion(),
-                    platform.system(),
-                    platform.machine(),
-                )
-            ))
-        ]
-        access_token = self._application.getCuraAPI().account.accessToken
-        if access_token:
-            self._request_headers.append((b"Authorization", "Bearer {}".format(access_token).encode()))
+    ##  Go back to the start state (welcome screen or loading if no login required)
+    def _restart(self):
+        # For an Essentials build, login is mandatory
+        if not self._application.getCuraAPI().account.isLoggedIn and ApplicationMetadata.IsEnterpriseVersion:
+            self.setViewPage("welcome")
+        else:
+            self.setViewPage("loading")
+            self._fetchPackageData()
 
     def _resetUninstallVariables(self) -> None:
         self._package_id_to_uninstall = None  # type: Optional[str]
@@ -152,29 +152,22 @@ class Toolbox(QObject, Extension):
 
     @pyqtSlot(str, int)
     def ratePackage(self, package_id: str, rating: int) -> None:
-        url = QUrl("{base_url}/packages/{package_id}/ratings".format(base_url=self._api_url, package_id = package_id))
-
-        self._rate_request = QNetworkRequest(url)
-        for header_name, header_value in self._request_headers:
-            cast(QNetworkRequest, self._rate_request).setRawHeader(header_name, header_value)
+        url = "{base_url}/packages/{package_id}/ratings".format(base_url = CloudApiModel.api_url, package_id = package_id)
         data = "{\"data\": {\"cura_version\": \"%s\", \"rating\": %i}}" % (Version(self._application.getVersion()), rating)
-        self._rate_reply = cast(QNetworkAccessManager, self._network_manager).put(self._rate_request, data.encode())
 
-    @pyqtSlot(result = str)
-    def getLicenseDialogPluginName(self) -> str:
-        return self._license_dialog_plugin_name
+        self._application.getHttpRequestManager().put(url, data = data.encode(), scope = self._json_scope)
 
-    @pyqtSlot(result = str)
     def getLicenseDialogPluginFileLocation(self) -> str:
         return self._license_dialog_plugin_file_location
 
-    @pyqtSlot(result = str)
-    def getLicenseDialogLicenseContent(self) -> str:
-        return self._license_dialog_license_content
+    def openLicenseDialog(self, plugin_name: str, license_content: str, plugin_file_location: str, icon_url: str) -> None:
+        # Set page 1/1 when opening the dialog for a single package
+        self._license_model.setCurrentPageIdx(0)
+        self._license_model.setPageCount(1)
+        self._license_model.setIconUrl(icon_url)
 
-    def openLicenseDialog(self, plugin_name: str, license_content: str, plugin_file_location: str) -> None:
-        self._license_dialog_plugin_name = plugin_name
-        self._license_dialog_license_content = license_content
+        self._license_model.setPackageName(plugin_name)
+        self._license_model.setLicenseText(license_content)
         self._license_dialog_plugin_file_location = plugin_file_location
         self.showLicenseDialog.emit()
 
@@ -183,11 +176,6 @@ class Toolbox(QObject, Extension):
     def _onAppInitialized(self) -> None:
         self._plugin_registry = self._application.getPluginRegistry()
         self._package_manager = self._application.getPackageManager()
-        self._api_url = "{cloud_api_root}/cura-packages/v{cloud_api_version}/cura/v{sdk_version}".format(
-            cloud_api_root = self._cloud_api_root,
-            cloud_api_version = self._cloud_api_version,
-            sdk_version = self._sdk_version
-        )
 
         # We need to construct a query like installed_packages=ID:VERSION&installed_packages=ID:VERSION, etc.
         installed_package_ids_with_versions = [":".join(items) for items in
@@ -195,40 +183,28 @@ class Toolbox(QObject, Extension):
         installed_packages_query = "&installed_packages=".join(installed_package_ids_with_versions)
 
         self._request_urls = {
-            "authors": QUrl("{base_url}/authors".format(base_url = self._api_url)),
-            "packages": QUrl("{base_url}/packages".format(base_url = self._api_url)),
-            "updates": QUrl("{base_url}/packages/package-updates?installed_packages={query}".format(
-                base_url = self._api_url, query = installed_packages_query))
+            "authors": "{base_url}/authors".format(base_url = CloudApiModel.api_url),
+            "packages": "{base_url}/packages".format(base_url = CloudApiModel.api_url),
+            "updates": "{base_url}/packages/package-updates?installed_packages={query}".format(
+                base_url = CloudApiModel.api_url, query = installed_packages_query)
         }
 
+        self._application.getCuraAPI().account.loginStateChanged.connect(self._restart)
+
         # On boot we check which packages have updates.
-        if len(installed_package_ids_with_versions) > 0:
-            self._fetchPackageUpdates()
+        if CuraApplication.getInstance().getPreferences().getValue("info/automatic_update_check") and len(installed_package_ids_with_versions) > 0:
+            # Request the latest and greatest!
+            self._makeRequestByType("updates")
 
-    def _prepareNetworkManager(self):
-        if self._network_manager is not None:
-            self._network_manager.finished.disconnect(self._onRequestFinished)
-            self._network_manager.networkAccessibleChanged.disconnect(self._onNetworkAccessibleChanged)
-        self._network_manager = QNetworkAccessManager()
-        self._network_manager.finished.connect(self._onRequestFinished)
-        self._network_manager.networkAccessibleChanged.connect(self._onNetworkAccessibleChanged)
 
-    def _fetchPackageUpdates(self):
-        self._prepareNetworkManager()
-        self._makeRequestByType("updates")
-
-    def _fetchPackageData(self):
-        self._prepareNetworkManager()
-        # Make remote requests:
+    def _fetchPackageData(self) -> None:
         self._makeRequestByType("packages")
         self._makeRequestByType("authors")
-        # Gather installed packages:
         self._updateInstalledModels()
 
+    # Displays the toolbox
     @pyqtSlot()
-    def browsePackages(self) -> None:
-        self._fetchPackageData()
-
+    def launch(self) -> None:
         if not self._dialog:
             self._dialog = self._createDialog("Toolbox.qml")
 
@@ -236,8 +212,9 @@ class Toolbox(QObject, Extension):
             Logger.log("e", "Unexpected error trying to create the 'Marketplace' dialog.")
             return
 
-        self._dialog.show()
+        self._restart()
 
+        self._dialog.show()
         # Apply enabled/disabled state to installed plugins
         self.enabledChanged.emit()
 
@@ -248,7 +225,11 @@ class Toolbox(QObject, Extension):
             return None
         path = os.path.join(plugin_path, "resources", "qml", qml_name)
 
-        dialog = self._application.createQmlComponent(path, {"toolbox": self})
+        dialog = self._application.createQmlComponent(path, {
+            "toolbox": self,
+            "handler": self,
+            "licenseModel": self._license_model
+        })
         if not dialog:
             raise Exception("Failed to create Marketplace dialog")
         return dialog
@@ -313,19 +294,22 @@ class Toolbox(QObject, Extension):
             self._old_plugin_metadata = {k: v for k, v in self._old_plugin_metadata.items() if k in self._old_plugin_ids}
 
             self._plugins_installed_model.setMetadata(all_packages["plugin"] + list(self._old_plugin_metadata.values()))
+            self._plugins_bundled_model.setMetadata(all_packages["plugin"] + list(self._old_plugin_metadata.values()))
             self.metadataChanged.emit()
         if "material" in all_packages:
             self._materials_installed_model.setMetadata(all_packages["material"])
+            self._materials_bundled_model.setMetadata(all_packages["material"])
             self.metadataChanged.emit()
 
     @pyqtSlot(str)
-    def install(self, file_path: str) -> None:
-        self._package_manager.installPackage(file_path)
+    def install(self, file_path: str) -> Optional[str]:
+        package_id = self._package_manager.installPackage(file_path)
         self.installChanged.emit()
         self._updateInstalledModels()
         self.metadataChanged.emit()
         self._restart_required = True
         self.restartRequiredChanged.emit()
+        return package_id
 
     ##  Check package usage and uninstall
     #   If the package is in use, you'll get a confirmation dialog to set everything to default
@@ -341,7 +325,7 @@ class Toolbox(QObject, Extension):
             self._package_used_qualities = package_used_qualities
             # Ask change to default material / profile
             if self._confirm_reset_dialog is None:
-                self._confirm_reset_dialog = self._createDialog("ToolboxConfirmUninstallResetDialog.qml")
+                self._confirm_reset_dialog = self._createDialog("dialogs/ToolboxConfirmUninstallResetDialog.qml")
             self.uninstallVariablesChanged.emit()
             if self._confirm_reset_dialog is None:
                 Logger.log("e", "ToolboxConfirmUninstallResetDialog should have been initialized, but it is not. Not showing dialog and not uninstalling package.")
@@ -397,6 +381,16 @@ class Toolbox(QObject, Extension):
         self._resetUninstallVariables()
         self.closeConfirmResetDialog()
 
+    @pyqtSlot()
+    def onLicenseAccepted(self):
+        self.closeLicenseDialog.emit()
+        package_id = self.install(self.getLicenseDialogPluginFileLocation())
+
+
+    @pyqtSlot()
+    def onLicenseDeclined(self):
+        self.closeLicenseDialog.emit()
+
     def _markPackageMaterialsAsToBeUninstalled(self, package_id: str) -> None:
         container_registry = self._application.getContainerRegistry()
 
@@ -526,9 +520,7 @@ class Toolbox(QObject, Extension):
 
     @pyqtSlot(str, result = bool)
     def isEnabled(self, package_id: str) -> bool:
-        if package_id in self._plugin_registry.getActivePlugins():
-            return True
-        return False
+        return package_id in self._plugin_registry.getActivePlugins()
 
     # Check for plugins that were installed with the old plugin browser
     def isOldPlugin(self, plugin_id: str) -> bool:
@@ -547,152 +539,136 @@ class Toolbox(QObject, Extension):
     # Make API Calls
     # --------------------------------------------------------------------------
     def _makeRequestByType(self, request_type: str) -> None:
-        Logger.log("d", "Requesting %s metadata from server.", request_type)
-        request = QNetworkRequest(self._request_urls[request_type])
-        for header_name, header_value in self._request_headers:
-            request.setRawHeader(header_name, header_value)
-        if self._network_manager:
-            self._network_manager.get(request)
+        Logger.log("d", "Requesting [%s] metadata from server.", request_type)
+        url = self._request_urls[request_type]
+
+        callback = lambda r, rt = request_type: self._onDataRequestFinished(rt, r)
+        error_callback = lambda r, e, rt = request_type: self._onDataRequestError(rt, r, e)
+        self._application.getHttpRequestManager().get(url,
+                                                      callback = callback,
+                                                      error_callback = error_callback,
+                                                      scope=self._json_scope)
 
     @pyqtSlot(str)
     def startDownload(self, url: str) -> None:
         Logger.log("i", "Attempting to download & install package from %s.", url)
-        url = QUrl(url)
-        self._download_request = QNetworkRequest(url)
-        if hasattr(QNetworkRequest, "FollowRedirectsAttribute"):
-            # Patch for Qt 5.6-5.8
-            cast(QNetworkRequest, self._download_request).setAttribute(QNetworkRequest.FollowRedirectsAttribute, True)
-        if hasattr(QNetworkRequest, "RedirectPolicyAttribute"):
-            # Patch for Qt 5.9+
-            cast(QNetworkRequest, self._download_request).setAttribute(QNetworkRequest.RedirectPolicyAttribute, True)
-        for header_name, header_value in self._request_headers:
-            cast(QNetworkRequest, self._download_request).setRawHeader(header_name, header_value)
-        self._download_reply = cast(QNetworkAccessManager, self._network_manager).get(self._download_request)
+
+        callback = lambda r: self._onDownloadFinished(r)
+        error_callback = lambda r, e: self._onDownloadFailed(r, e)
+        download_progress_callback = self._onDownloadProgress
+        request_data = self._application.getHttpRequestManager().get(url,
+                                                                     callback = callback,
+                                                                     error_callback = error_callback,
+                                                                     download_progress_callback = download_progress_callback,
+                                                                     scope=self._cloud_scope
+                                                                     )
+
+        self._download_request_data = request_data
         self.setDownloadProgress(0)
         self.setIsDownloading(True)
-        cast(QNetworkReply, self._download_reply).downloadProgress.connect(self._onDownloadProgress)
 
     @pyqtSlot()
     def cancelDownload(self) -> None:
-        Logger.log("i", "User cancelled the download of a package.")
+        Logger.log("i", "User cancelled the download of a package. request %s", self._download_request_data)
+        if self._download_request_data is not None:
+            self._application.getHttpRequestManager().abortRequest(self._download_request_data)
+            self._download_request_data = None
         self.resetDownload()
 
     def resetDownload(self) -> None:
-        if self._download_reply:
-            try:
-                self._download_reply.downloadProgress.disconnect(self._onDownloadProgress)
-            except (TypeError, RuntimeError):  # Raised when the method is not connected to the signal yet.
-                pass  # Don't need to disconnect.
-            try:
-                self._download_reply.abort()
-            except RuntimeError:
-                # In some cases the garbage collector is a bit to agressive, which causes the dowload_reply
-                # to be deleted (especially if the machine has been put to sleep). As we don't know what exactly causes
-                # this (The issue probably lives in the bowels of (py)Qt somewhere), we can only catch and ignore it.
-                pass
-        self._download_reply = None
-        self._download_request = None
         self.setDownloadProgress(0)
         self.setIsDownloading(False)
 
     # Handlers for Network Events
     # --------------------------------------------------------------------------
-    def _onNetworkAccessibleChanged(self, network_accessibility: QNetworkAccessManager.NetworkAccessibility) -> None:
-        if network_accessibility == QNetworkAccessManager.NotAccessible:
-            self.resetDownload()
+    def _onDataRequestError(self, request_type: str, reply: "QNetworkReply", error: "QNetworkReply.NetworkError") -> None:
+        Logger.log("e", "Request [%s] failed due to error [%s]: %s", request_type, error, reply.errorString())
+        self.setViewPage("errored")
 
-    def _onRequestFinished(self, reply: QNetworkReply) -> None:
-        if reply.error() == QNetworkReply.TimeoutError:
-            Logger.log("w", "Got a timeout.")
-            self.setViewPage("errored")
-            self.resetDownload()
+    def _onDataRequestFinished(self, request_type: str, reply: "QNetworkReply") -> None:
+        if reply.operation() != QNetworkAccessManager.GetOperation:
+            Logger.log("e", "_onDataRequestFinished() only handles GET requests but got [%s] instead", reply.operation())
             return
 
-        if reply.error() == QNetworkReply.HostNotFoundError:
-            Logger.log("w", "Unable to reach server.")
+        http_status_code = reply.attribute(QNetworkRequest.HttpStatusCodeAttribute)
+        if http_status_code != 200:
+            Logger.log("e", "Request type [%s] got non-200 HTTP response: [%s]", http_status_code)
             self.setViewPage("errored")
-            self.resetDownload()
             return
 
-        if reply.operation() == QNetworkAccessManager.GetOperation:
-            for response_type, url in self._request_urls.items():
-                if reply.url() == url:
-                    if reply.attribute(QNetworkRequest.HttpStatusCodeAttribute) == 200:
-                        try:
-                            json_data = json.loads(bytes(reply.readAll()).decode("utf-8"))
+        data = bytes(reply.readAll())
+        try:
+            json_data = json.loads(data.decode("utf-8"))
+        except json.decoder.JSONDecodeError:
+            Logger.log("e", "Failed to decode response data as JSON for request type [%s], response data [%s]",
+                       request_type, data)
+            self.setViewPage("errored")
+            return
 
-                            # Check for errors:
-                            if "errors" in json_data:
-                                for error in json_data["errors"]:
-                                    Logger.log("e", "%s", error["title"])
-                                return
+        # Check for errors:
+        if "errors" in json_data:
+            for error in json_data["errors"]:
+                Logger.log("e", "Request type [%s] got response showing error: %s", error["title"])
+            self.setViewPage("errored")
+            return
 
-                            # Create model and apply metadata:
-                            if not self._models[response_type]:
-                                Logger.log("e", "Could not find the %s model.", response_type)
-                                break
+        # Create model and apply metadata:
+        if not self._models[request_type]:
+            Logger.log("e", "Could not find the model for request type [%s].", request_type)
+            self.setViewPage("errored")
+            return
 
-                            self._server_response_data[response_type] = json_data["data"]
-                            self._models[response_type].setMetadata(self._server_response_data[response_type])
+        self._server_response_data[request_type] = json_data["data"]
+        self._models[request_type].setMetadata(self._server_response_data[request_type])
 
-                            if response_type == "packages":
-                                self._models[response_type].setFilter({"type": "plugin"})
-                                self.reBuildMaterialsModels()
-                                self.reBuildPluginsModels()
-                                self._notifyPackageManager()
-                            elif response_type == "authors":
-                                self._models[response_type].setFilter({"package_types": "material"})
-                                self._models[response_type].setFilter({"tags": "generic"})
-                            elif response_type == "updates":
-                                # Tell the package manager that there's a new set of updates available.
-                                packages = set([pkg["package_id"] for pkg in self._server_response_data[response_type]])
-                                self._package_manager.setPackagesWithUpdate(packages)
+        if request_type == "packages":
+            self._models[request_type].setFilter({"type": "plugin"})
+            self.reBuildMaterialsModels()
+            self.reBuildPluginsModels()
+            self._notifyPackageManager()
+        elif request_type == "authors":
+            self._models[request_type].setFilter({"package_types": "material"})
+            self._models[request_type].setFilter({"tags": "generic"})
+        elif request_type == "updates":
+            # Tell the package manager that there's a new set of updates available.
+            packages = set([pkg["package_id"] for pkg in self._server_response_data[request_type]])
+            self._package_manager.setPackagesWithUpdate(packages)
 
-                            self.metadataChanged.emit()
+        self.metadataChanged.emit()
 
-                            if self.isLoadingComplete():
-                                self.setViewPage("overview")
-
-                        except json.decoder.JSONDecodeError:
-                            Logger.log("w", "Received invalid JSON for %s.", response_type)
-                            break
-                    else:
-                        Logger.log("w", "Unable to connect with the server, we got a response code %s while trying to connect to %s", reply.attribute(QNetworkRequest.HttpStatusCodeAttribute), reply.url())
-                        self.setViewPage("errored")
-                        self.resetDownload()
-        elif reply.operation() == QNetworkAccessManager.PutOperation:
-            # Ignore any operation that is not a get operation
-            pass
+        if self.isLoadingComplete():
+            self.setViewPage("overview")
 
     # This function goes through all known remote versions of a package and notifies the package manager of this change
     def _notifyPackageManager(self):
         for package in self._server_response_data["packages"]:
             self._package_manager.addAvailablePackageVersion(package["package_id"], Version(package["package_version"]))
 
+    def _onDownloadFinished(self, reply: "QNetworkReply") -> None:
+        self.resetDownload()
+
+        if reply.attribute(QNetworkRequest.HttpStatusCodeAttribute) != 200:
+            Logger.log("w", "Failed to download package. The following error was returned: %s",
+                       json.loads(reply.readAll().data().decode("utf-8")))
+            return
+        # Must not delete the temporary file on Windows
+        self._temp_plugin_file = tempfile.NamedTemporaryFile(mode = "w+b", suffix = ".curapackage", delete = False)
+        file_path = self._temp_plugin_file.name
+        # Write first and close, otherwise on Windows, it cannot read the file
+        self._temp_plugin_file.write(reply.readAll())
+        self._temp_plugin_file.close()
+        self._onDownloadComplete(file_path)
+
+    def _onDownloadFailed(self, reply: "QNetworkReply", error: "QNetworkReply.NetworkError") -> None:
+        Logger.log("w", "Failed to download package. The following error was returned: %s", error)
+
+        self.resetDownload()
+
     def _onDownloadProgress(self, bytes_sent: int, bytes_total: int) -> None:
         if bytes_total > 0:
             new_progress = bytes_sent / bytes_total * 100
             self.setDownloadProgress(new_progress)
-            if bytes_sent == bytes_total:
-                self.setIsDownloading(False)
-                self._download_reply = cast(QNetworkReply, self._download_reply)
-                self._download_reply.downloadProgress.disconnect(self._onDownloadProgress)
-
-                # Check if the download was sucessfull
-                if self._download_reply.attribute(QNetworkRequest.HttpStatusCodeAttribute) != 200:
-                    try:
-                        Logger.log("w", "Failed to download package. The following error was returned: %s", json.loads(bytes(self._download_reply.readAll()).decode("utf-8")))
-                    except json.decoder.JSONDecodeError:
-                        Logger.logException("w", "Failed to download package and failed to parse a response from it")
-                    finally:
-                        return
-                # Must not delete the temporary file on Windows
-                self._temp_plugin_file = tempfile.NamedTemporaryFile(mode = "w+b", suffix = ".curapackage", delete = False)
-                file_path = self._temp_plugin_file.name
-                # Write first and close, otherwise on Windows, it cannot read the file
-                self._temp_plugin_file.write(cast(QNetworkReply, self._download_reply).readAll())
-                self._temp_plugin_file.close()
-                self._onDownloadComplete(file_path)
+            Logger.log("d", "new download progress %s / %s : %s%%", bytes_sent, bytes_total, new_progress)
 
     def _onDownloadComplete(self, file_path: str) -> None:
         Logger.log("i", "Download complete.")
@@ -702,11 +678,16 @@ class Toolbox(QObject, Extension):
             return
 
         license_content = self._package_manager.getPackageLicense(file_path)
+        package_id = package_info["package_id"]
         if license_content is not None:
-            self.openLicenseDialog(package_info["package_id"], license_content, file_path)
+            # get the icon url for package_id, make sure the result is a string, never None
+            icon_url = next((x["icon_url"] for x in self.packagesModel.items if x["id"] == package_id), None) or ""
+            self.openLicenseDialog(package_info["display_name"], license_content, file_path, icon_url)
             return
 
-        self.install(file_path)
+        installed_id = self.install(file_path)
+        if installed_id != package_id:
+            Logger.error("Installed package {} does not match {}".format(installed_id, package_id))
 
     # Getter & Setters for Properties:
     # --------------------------------------------------------------------------
@@ -728,14 +709,14 @@ class Toolbox(QObject, Extension):
     def isDownloading(self) -> bool:
         return self._is_downloading
 
-    def setActivePackage(self, package: Dict[str, Any]) -> None:
+    def setActivePackage(self, package: QObject) -> None:
         if self._active_package != package:
             self._active_package = package
             self.activePackageChanged.emit()
 
     ##  The active package is the package that is currently being downloaded
     @pyqtProperty(QObject, fset = setActivePackage, notify = activePackageChanged)
-    def activePackage(self) -> Optional[Dict[str, Any]]:
+    def activePackage(self) -> Optional[QObject]:
         return self._active_package
 
     def setViewCategory(self, category: str = "plugin") -> None:
@@ -743,6 +724,11 @@ class Toolbox(QObject, Extension):
             self._view_category = category
             self.viewChanged.emit()
 
+    ## Function explicitly defined so that it can be called through the callExtensionsMethod
+    # which cannot receive arguments.
+    def setViewCategoryToMaterials(self) -> None:
+        self.setViewCategory("material")
+
     @pyqtProperty(str, fset = setViewCategory, notify = viewChanged)
     def viewCategory(self) -> str:
         return self._view_category
@@ -758,42 +744,57 @@ class Toolbox(QObject, Extension):
 
     # Exposed Models:
     # --------------------------------------------------------------------------
-    @pyqtProperty(QObject, constant=True)
+    @pyqtProperty(QObject, constant = True)
     def authorsModel(self) -> AuthorsModel:
         return cast(AuthorsModel, self._models["authors"])
 
-    @pyqtProperty(QObject, constant=True)
+    @pyqtProperty(QObject, constant = True)
     def packagesModel(self) -> PackagesModel:
         return cast(PackagesModel, self._models["packages"])
 
-    @pyqtProperty(QObject, constant=True)
+    @pyqtProperty(QObject, constant = True)
     def pluginsShowcaseModel(self) -> PackagesModel:
         return self._plugins_showcase_model
 
-    @pyqtProperty(QObject, constant=True)
+    @pyqtProperty(QObject, constant = True)
     def pluginsAvailableModel(self) -> PackagesModel:
         return self._plugins_available_model
 
-    @pyqtProperty(QObject, constant=True)
+    @pyqtProperty(QObject, constant = True)
     def pluginsInstalledModel(self) -> PackagesModel:
         return self._plugins_installed_model
 
-    @pyqtProperty(QObject, constant=True)
+    @pyqtProperty(QObject, constant = True)
+    def pluginsBundledModel(self) -> PackagesModel:
+        return self._plugins_bundled_model
+
+    @pyqtProperty(QObject, constant = True)
     def materialsShowcaseModel(self) -> AuthorsModel:
         return self._materials_showcase_model
 
-    @pyqtProperty(QObject, constant=True)
+    @pyqtProperty(QObject, constant = True)
     def materialsAvailableModel(self) -> AuthorsModel:
         return self._materials_available_model
 
-    @pyqtProperty(QObject, constant=True)
+    @pyqtProperty(QObject, constant = True)
     def materialsInstalledModel(self) -> PackagesModel:
         return self._materials_installed_model
 
-    @pyqtProperty(QObject, constant=True)
+    @pyqtProperty(QObject, constant = True)
+    def materialsBundledModel(self) -> PackagesModel:
+        return self._materials_bundled_model
+
+    @pyqtProperty(QObject, constant = True)
     def materialsGenericModel(self) -> PackagesModel:
         return self._materials_generic_model
 
+    @pyqtSlot(str, result = str)
+    def getWebMarketplaceUrl(self, page: str) -> str:
+        root = CuraMarketplaceRoot
+        if root == "":
+            root = DEFAULT_MARKETPLACE_ROOT
+        return root + "/app/cura/" + page
+
     # Filter Models:
     # --------------------------------------------------------------------------
     @pyqtSlot(str, str, str)
